@@ -103,7 +103,7 @@ static int
 get_baud_rate(int rate, int *val, int cisco, int *bps)
 {
     unsigned int i;
-    for (i=0; i<BAUD_RATES_LEN; i++) {
+    for (i = 0; i < BAUD_RATES_LEN; i++) {
 	if (cisco) {
 	    if (rate == baud_rates[i].cisco_ios_val) {
 		*val = baud_rates[i].val;
@@ -126,7 +126,7 @@ static void
 get_rate_from_baud_rate(int baud_rate, int *val, int cisco)
 {
     unsigned int i;
-    for (i=0; i<BAUD_RATES_LEN; i++) {
+    for (i = 0; i < BAUD_RATES_LEN; i++) {
 	if (baud_rate == baud_rates[i].val) {
 	    if (cisco) {
 		if (baud_rates[i].cisco_ios_val < 0)
@@ -144,35 +144,37 @@ get_rate_from_baud_rate(int baud_rate, int *val, int cisco)
 }
 
 #ifdef USE_UUCP_LOCKING
-static char *uucp_lck_dir = "/var/lock";
+static char *uucp_lck_dir = "/var/lock/";
+static char *dev_prefix = "/dev/";
 
 static int
 uucp_fname_lock_size(char *devname)
 {
-    char *ptr;
+    int dev_prefix_len = strlen(dev_prefix);
 
-    (ptr = strrchr(devname, '/'));
-    if (ptr == NULL) {
-	ptr = devname;
-    } else {
-	ptr = ptr + 1;
-    }
+    if (strncmp(dev_prefix, devname, dev_prefix_len) == 0)
+	devname += dev_prefix_len;
 
-    return 7 + strlen(uucp_lck_dir) + strlen(ptr);
+    /*
+     * Format is "/var/lock/LCK..<devname>".  The 6 is for
+     * the "LCK.." and the final nil char.
+     */
+    return 6 + strlen(uucp_lck_dir) + strlen(devname);
 }
 
 static void
 uucp_fname_lock(char *buf, char *devname)
 {
-    char *ptr;
+    int i, dev_prefix_len = strlen(dev_prefix);
 
-    (ptr = strrchr(devname, '/'));
-    if (ptr == NULL) {
-	ptr = devname;
-    } else {
-	ptr = ptr + 1;
+    if (strncmp(dev_prefix, devname, dev_prefix_len) == 0)
+	devname += dev_prefix_len;
+
+    sprintf(buf, "%sLCK..%s", uucp_lck_dir, devname);
+    for (i = strlen(uucp_lck_dir); buf[i]; i++) {
+	if (buf[i] == '/')
+	    buf[i] = '_';
     }
-    sprintf(buf, "%s/LCK..%s", uucp_lck_dir, ptr);
 }
 
 static void
@@ -225,7 +227,7 @@ uucp_mk_lock(char *devname)
 		pid = buf.ival;
 	    else if (n > 0) {		/* Ascii lockfile. */
 		buf.str[n] = 0;
-		sscanf(buf.str, "%d", &pid);
+		sscanf(buf.str, "%10d", &pid);
 	    }
 
 	    if (pid > 0 && kill((pid_t)pid, 0) < 0 && errno == ESRCH) {
@@ -565,11 +567,11 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
         } else if (strcmp(pos, "-RTSCTS") == 0) {
 	    set_termios_rtscts(termctl, 0);
         } else if (strcmp(pos, "LOCAL") == 0) {
-            termctl->c_cflag |= CLOCAL;  
+            termctl->c_cflag |= CLOCAL;
         } else if (strcmp(pos, "-LOCAL") == 0) {
             termctl->c_cflag &= ~CLOCAL;
         } else if (strcmp(pos, "HANGUP_WHEN_DONE") == 0) {
-            termctl->c_cflag |= HUPCL;  
+            termctl->c_cflag |= HUPCL;
         } else if (strcmp(pos, "-HANGUP_WHEN_DONE") == 0) {
             termctl->c_cflag &= ~HUPCL;
 	} else if (strcmp(pos, "NOBREAK") == 0) {
@@ -666,7 +668,7 @@ devcfg_serparm_to_str(struct devio *io, char *str, int strlen)
 
     sstr = baud_string(speed);
 
-    if (stopbits) 
+    if (stopbits)
 	schar = '2';
     else
 	schar = '1';
@@ -696,7 +698,7 @@ devcfg_show_devcfg(struct devio *io, struct absout *out)
 {
     struct devcfg_data *d = io->my_data;
     struct termios *termctl = &d->termctl;
-    
+
     speed_t speed = cfgetospeed(termctl);
     int     stopbits = termctl->c_cflag & CSTOPB;
     int     databits = termctl->c_cflag & CSIZE;
@@ -714,8 +716,8 @@ devcfg_show_devcfg(struct devio *io, struct absout *out)
 
     if (xon && xoff && xany) {
       out->out(out, "XONXOFF ");
-    }      
-    
+    }
+
     if (flow_rtscts) {
       out->out(out, "RTSCTS ");
     }
@@ -860,7 +862,14 @@ devfd_fd_cleared(int fd, void *cb_data)
     struct devio *io = cb_data;
     struct devcfg_data *d = io->my_data;
     void (*shutdown_done)(struct devio *) = d->shutdown_done;
+    struct termios termio;
 
+    /* Disable flow control to avoid a long shutdown. */
+    if (tcgetattr(d->devfd, &termio) != -1) {
+	termio.c_iflag &= ~(IXON | IXOFF);
+	termio.c_cflag &= ~CRTSCTS;
+	tcsetattr(d->devfd, TCSANOW, &termio);
+    }
     tcflush(d->devfd, TCOFLUSH);
     close(d->devfd);
     d->devfd = -1;
@@ -890,7 +899,7 @@ static int devcfg_setup(struct devio *io, const char *name, const char **errstr,
     d->current_parity_on = d->default_parity_on;
     *bpc = calc_bpc(d);
 
-    /* Oct 05 2001 druzus: NOCTTY - don't make 
+    /* Oct 05 2001 druzus: NOCTTY - don't make
        device control tty for our process */
     options = O_NONBLOCK | O_NOCTTY;
     if (io->read_disabled) {
@@ -1235,7 +1244,7 @@ static int devcfg_control(struct devio *io, unsigned char *val)
 	if (ioctl(d->devfd, TIOCSBRK) != -1)
 	    d->break_set = 1;
 	goto read_break_val;
-	    
+
     case 4:
     read_break_val:
 	if (d->break_set)
@@ -1266,7 +1275,7 @@ static int devcfg_control(struct devio *io, unsigned char *val)
 	ioctl(d->devfd, TIOCMSET, &ival);
 #endif
 	goto read_dtr_val;
-	    
+
     case 7:
     read_dtr_val:
 	if (ioctl(d->devfd, TIOCMGET, &ival) == -1)
@@ -1288,7 +1297,7 @@ static int devcfg_control(struct devio *io, unsigned char *val)
 	ioctl(d->devfd, TIOCMSET, &ival);
 #endif
 	goto read_rts_val;
-	
+
     case 12:
 #ifndef __CYGWIN__
 	ival = TIOCM_RTS;
@@ -1299,7 +1308,7 @@ static int devcfg_control(struct devio *io, unsigned char *val)
 	ioctl(d->devfd, TIOCMSET, &ival);
 #endif
 	goto read_rts_val;
-	    
+
     case 10:
     read_rts_val:
 	if (ioctl(d->devfd, TIOCMGET, &ival) == -1)
