@@ -177,8 +177,8 @@ struct selector_s
        moderately wasteful of space, but easy to do.  Hey, memory is
        cheap. */
     volatile fd_control_t fds[FD_SETSIZE];
-    
-    /* These are the offical fd_sets used to track what file descriptors
+
+  /* These are the offical fd_sets used to track what file descriptors
        need to be monitored. */
     volatile fd_set read_set;
     volatile fd_set write_set;
@@ -627,7 +627,6 @@ int
 sel_stop_timer(sel_timer_t *timer)
 {
     selector_t *sel = timer->val.sel;
-    volatile sel_timer_t *top;
 
     sel_timer_lock(sel);
     if (timer->val.stopped) {
@@ -636,7 +635,7 @@ sel_stop_timer(sel_timer_t *timer)
     }
 
     if (timer->val.in_heap) {
-	top = theap_get_top(&sel->timer_heap);
+	volatile sel_timer_t *top = theap_get_top(&sel->timer_heap);
 
 	theap_remove(&sel->timer_heap, timer);
 	timer->val.in_heap = 0;
@@ -885,7 +884,7 @@ process_fds(selector_t	            *sel,
     memcpy(&tmp_read_set, (void *) &sel->read_set, sizeof(tmp_read_set));
     memcpy(&tmp_write_set, (void *) &sel->write_set, sizeof(tmp_write_set));
     memcpy(&tmp_except_set, (void *) &sel->except_set, sizeof(tmp_except_set));
-    num_fds = sel->maxfd+1;
+    num_fds = sel->maxfd + 1;
     sel_fd_unlock(sel);
 
     err = select(num_fds,
@@ -1009,10 +1008,9 @@ sel_select_loop(selector_t      *sel,
 		long            thread_id,
 		void            *cb_data)
 {
-    int             err;
-
     for (;;) {
-	err = sel_select(sel, send_sig, thread_id, cb_data, NULL);
+	int err = sel_select(sel, send_sig, thread_id, cb_data, NULL);
+
 	if ((err < 0) && (errno != EINTR)) {
 	    err = errno;
 	    /* An error occurred. */
@@ -1055,7 +1053,7 @@ sel_alloc_selector_thread(selector_t **new_selector, int wake_sig,
     FD_ZERO((fd_set *) &sel->write_set);
     FD_ZERO((fd_set *) &sel->except_set);
 
-    for (i=0; i<FD_SETSIZE; i++) {
+    for (i = 0; i < FD_SETSIZE; i++) {
 	init_fd((fd_control_t *) &(sel->fds[i]));
     }
 
