@@ -509,7 +509,7 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    for (i=1; i<argc; i++) {
+    for (i = 1; i < argc; i++) {
 	if ((argv[i][0] != '-') || (strlen(argv[i]) != 2)) {
 	    fprintf(stderr, "Invalid argument: '%s'\n", argv[i]);
 	    arg_error(argv[0]);
@@ -539,7 +539,7 @@ main(int argc, char *argv[])
 		fprintf(stderr, "No config line specified with -C\n");
 		arg_error(argv[0]);
 	    }
-	    handle_config_line(argv[i]);
+	    handle_config_line(argv[i], strlen(argv[i]));
 	    config_file = NULL;
 	    break;
 
@@ -567,7 +567,7 @@ main(int argc, char *argv[])
 	    }
 	    config_port_from_cmdline = 1;
 	    break;
-	
+
 	case 'P':
 	    i++;
 	    if (i == argc) {
@@ -618,9 +618,15 @@ main(int argc, char *argv[])
 	}
     }
 
-    err = sel_alloc_selector_thread(&ser2net_sel, ser2net_wake_sig,
-				    slock_alloc, slock_free,
-				    slock_lock, slock_unlock, NULL);
+#ifdef USE_PTHREADS
+    if (num_threads > 1)
+	err = sel_alloc_selector_thread(&ser2net_sel, ser2net_wake_sig,
+					slock_alloc, slock_free,
+					slock_lock, slock_unlock, NULL);
+    else
+#endif
+	err = sel_alloc_selector_nothread(&ser2net_sel);
+
     if (err) {
 	fprintf(stderr,
 		"Could not initialize ser2net selector: '%s'\n",
@@ -702,5 +708,7 @@ main(int argc, char *argv[])
     start_threads();
     op_loop(NULL);
 
+    sel_free_selector(ser2net_sel);
+
     return 0;
 }
