@@ -27,4 +27,12 @@ extern char *rfc2217_signature;
 
 extern selector_t *ser2net_sel;
 
+extern int ser2net_debug;
+extern int ser2net_debug_level;
+
+extern int ser2net_wake_sig;
+
+void start_maint_op(void);
+void end_maint_op(void);
+
 #endif /* SER2NET_H */
