@@ -161,11 +161,11 @@ baud_string(int speed)
 {
     char *str;
     switch (speed) {
-    case IPMI_SOL_BIT_RATE_9600: str = "9600";
-    case IPMI_SOL_BIT_RATE_19200: str = "19200";
-    case IPMI_SOL_BIT_RATE_38400: str = "38400";
-    case IPMI_SOL_BIT_RATE_57600: str = "57600";
-    case IPMI_SOL_BIT_RATE_115200: str = "115200";
+    case IPMI_SOL_BIT_RATE_9600: str = "9600"; break;
+    case IPMI_SOL_BIT_RATE_19200: str = "19200"; break;
+    case IPMI_SOL_BIT_RATE_38400: str = "38400"; break;
+    case IPMI_SOL_BIT_RATE_57600: str = "57600"; break;
+    case IPMI_SOL_BIT_RATE_115200: str = "115200"; break;
     default: str = "unknown speed";
     }
     return str;
@@ -184,7 +184,7 @@ static void
 solcfg_show_solcfg(struct devio *io, struct absout *out)
 {
     struct solcfg_data *d = io->my_data;
-    
+
     out->out(out, "%s", baud_string(d->speed));
 }
 
@@ -229,10 +229,6 @@ static int solcfg_read(struct devio *io, void *buf, size_t size)
 
     if (size == 0)
 	return 0;
-    if (size < 0) {
-	errno = EINVAL;
-	return -1;
-    }
 
     if (d->read_end == d->read_start) {
 	errno = EAGAIN;
@@ -319,13 +315,9 @@ static int solcfg_write(struct devio *io, void *buf, size_t size)
 	errno = EAGAIN;
 	return -1;
     }
-    
+
     if (size == 0)
 	return 0;
-    if (size < 0) {
-	errno = EINVAL;
-	return -1;
-    }
 
     tc = malloc(sizeof(*tc));
     if (!tc) {
@@ -447,9 +439,10 @@ static int solcfg_flush(struct devio *io, int *val)
 }
 
 static int sol_data_received(ipmi_sol_conn_t *conn,
-			     const void *data, size_t count, void *user_data)
+			     const void *idata, size_t count, void *user_data)
 {
     struct solcfg_data *d = user_data;
+    const char *data = idata;
     int space;
 
     if (d->read_end >= d->read_start)
@@ -473,10 +466,10 @@ static int sol_data_received(ipmi_sol_conn_t *conn,
 	count -= space;
 	d->read_end = 0;
     }
-	
+
     memcpy(d->read_data + d->read_end, data, count);
     d->read_end += count;
-    
+
     check_read_handler(d);
 
     return 0;
@@ -496,7 +489,8 @@ static void bmc_transmit_overrun(ipmi_sol_conn_t *conn, void *user_data)
 
 static void finish_close_connection(struct solcfg_data *d)
 {
-printf("Finish close: %d %p\n", d->closed, d->ipmi);
+    if (ser2net_debug_level > 0)
+	printf("Finish close: %d %p\n", d->closed, d->ipmi);
     if (d->sol)
 	ipmi_sol_free(d->sol);
     d->sol = NULL;
@@ -512,7 +506,8 @@ static void sol_connection_state(ipmi_sol_conn_t *conn, ipmi_sol_state state,
 {
     struct solcfg_data *d = cb_data;
 
-printf("sol_con_change: %d %d\n", state, error);
+    if (ser2net_debug_level > 0)
+	printf("sol_con_change: %d %d\n", state, error);
     if (error) {
 	finish_close_connection(d);
 	if (d->io)
@@ -521,7 +516,7 @@ printf("sol_con_change: %d %d\n", state, error);
 	    free(d);
 	return;
     }
-	
+
     switch (state) {
     case ipmi_sol_state_closed:
 	finish_close_connection(d);
@@ -555,7 +550,8 @@ static void conn_changed(ipmi_con_t   *ipmi,
 {
     struct solcfg_data *d = cb_data;
 
-printf("con_change: %d %d\n", any_port_up, err);
+    if (ser2net_debug_level > 0)
+	printf("con_change: %d %d\n", any_port_up, err);
     if (any_port_up == d->last_any_port_up)
 	return;
 
@@ -641,7 +637,7 @@ static int solcfg_setup(struct devio *io, const char *name, const char **errstr,
 					 d->deassert_CTS_DCD_DSR_on_connect);
 
     ipmi_sol_set_bit_rate(d->sol, d->speed);
-    
+
     *bps = get_rate_from_sol_baud_rate(d->speed, 0);
     *bpc = 10;
 
@@ -732,7 +728,7 @@ solconfig(struct solcfg_data *d, struct absout *eout, const char *instr,
 		  d->io->devname + 4, speed);
 	return -1;
     }
-    
+
     /* Enable authentication and encryption by default. */
     d->authenticated = find_default_int("authenticated");
     d->encrypted = find_default_int("encrypted");
@@ -751,7 +747,7 @@ solconfig(struct solcfg_data *d, struct absout *eout, const char *instr,
 	eout->out(eout, "Invalid SOL string: %s", d->io->devname + 4);
 	return -1;
     }
-    
+
     rv = ipmi_parse_args2(&curr_arg, argc, argv, &args);
     if (rv) {
 	str_to_argv_free(argc, argv);
@@ -834,7 +830,7 @@ solconfig(struct solcfg_data *d, struct absout *eout, const char *instr,
     if (rv && args)
 	ipmi_free_args(args);
     if (!rv) {
-	if (d->args) 
+	if (d->args)
 	    ipmi_free_args(d->args);
 	d->args = args;
     }
@@ -884,7 +880,7 @@ sol_ipmi_log(os_handler_t *hnd, enum ipmi_log_type_e log_type,
 	     const char *format, va_list ap)
 {
     int slevel = -1;
-    
+
     switch (log_type) {
     case IPMI_LOG_INFO:
 	if (ser2net_debug_level > 0)
